"use strict";
// Licensed Materials - Property of IBM
//
// IBM Watson Analytics
//
// (C) Copyright IBM Corp. 2019, 2020
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
Object.defineProperty(exports, "__esModule", { value: true });
exports.asModule = exports.getJSONFromXML = exports.isValidXML = void 0;
const xml2js_1 = require("xml2js");
/**
 * Checks if an XML string has valid XML syntax.
 * @param _xmlString A string containing XML content.
 * @returns A promise that resolves with true if the XML content is valid. If the
 * XML content is not valid, the promise is resolved with false.
 */
function isValidXML(_xmlString) {
    return new Promise(_resolve => {
        xml2js_1.parseString(_xmlString, (_err, _result) => {
            if (_err)
                return _resolve(false);
            if (_result)
                return _resolve(true);
            return _resolve(false);
        });
    });
}
exports.isValidXML = isValidXML;
/**
 * Converts an XML string to a JSON object.
 * @param _xmlString A string containing XML content.
 * @returns A promise that resolves with an object representing the XML. If the
 * XML content is invalid, the promise is rejected.
 */
function getJSONFromXML(_xmlString) {
    return new Promise((_resolve, _reject) => {
        xml2js_1.parseString(_xmlString, (_err, _result) => {
            if (_err)
                return _reject(_err);
            if (_result)
                return _resolve(_result);
            return _reject();
        });
    });
}
exports.getJSONFromXML = getJSONFromXML;
/**
 * Regular expression to escape characters for safe embedding into a string.
 */
const reEscape = /['\\\b\f\n\r\t\u2028\u2029]/g;
/**
 * Method that will return the escaped representation of `_char` for embedding it in a single quoted string.
 * Used as replace method for `reEscape`, so keep it in sync with the regular expression!
 * @param _char The character to escape.
 * @returns The escaped representation.
 */
function escapeChar(_char) {
    // using charCodeAt (instead of codePointAt) is safe,
    // no non-BMP characters are to be replaced
    switch (_char.charCodeAt(0)) {
        case 8: return "\\b";
        case 9: return "\\t";
        case 10: return "\\n";
        case 12: return "\\f";
        case 13: return "\\r";
        case 39: return "\\'";
        case 92: return "\\\\";
        case 8232: return "\\u2028";
        case 8233: return "\\u2029";
        default:
            throw new Error(`Unhandled character: "${_char}"`);
    }
}
/**
 * Converts an XML string to a string that represents a JavaScript module that returns
 * that XML string. Typically this function is used when creating manifest.xml.js or
 * vizdef.xml.js.
 * @param _content A string containing XML content.
 * @returns A string representing a JavaScript module that returns the XML content.
 */
function asModule(_content) {
    const escaped = String(_content).replace(reEscape, escapeChar);
    return `define( [], function () { return '${escaped}';});\n`;
}
exports.asModule = asModule;
